from django.core.urlresolvers import reverse_lazy as reverse
from django.utils.translation import ugettext_lazy as _

from ..forms.participant import ParticipantForm
from ..models.roles import Participant
from ..utils import reverse_with_back
from .generic import CreateView, ListView, UpdateView


class ParticipantListView(ListView):
    add_label           = _('add participant')
    model               = Participant
    template_name       = 'leprikon/participants.html'
    preview_template    = 'leprikon/participant_preview.html'

    def get_title(self):
        return _('Participants and parents')

    def get_queryset(self):
        return self.request.user.leprikon_participants.all()

    def get_add_url(self):
        return reverse_with_back(self.request, 'leprikon:participant_create')



class ParticipantCreateView(CreateView):
    model       = Participant
    form_class  = ParticipantForm
    success_url = reverse('leprikon:summary')
    title       = _('New participant')

    def get_form_kwargs(self):
        kwargs = super(ParticipantCreateView, self).get_form_kwargs()
        kwargs['user'] = self.request.user
        parent = self.request.user.leprikon_parents.first()
        if parent:
            kwargs['initial'] = dict((attr, getattr(parent, attr)) for attr in ['street', 'city', 'postal_code'])
        return kwargs

    def get_message(self):
        return _('New participant {} has been created.').format(self.object)



class ParticipantUpdateView(UpdateView):
    model       = Participant
    form_class  = ParticipantForm
    success_url = reverse('leprikon:summary')
    title       = _('Change participant')

    def get_queryset(self):
        # only allow to edit user's own participants
        return super(ParticipantUpdateView, self).get_queryset().filter(user=self.request.user)

    def get_form_kwargs(self):
        kwargs = super(ParticipantUpdateView, self).get_form_kwargs()
        kwargs['user'] = self.request.user
        return kwargs

    def get_message(self):
        return _('The participant {} has been updated.').format(self.object)



# Generated by Django 2.2 on 2019-05-05 08:56

from django.db import migrations, models
import django.db.models.deletion
import django_pays.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Gateway',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, verbose_name='name')),
                ('slug', models.SlugField(max_length=64, verbose_name='slug')),
                ('merchant_id', models.CharField(max_length=64, verbose_name='merchant identifier')),
                ('shop_id', models.CharField(max_length=64, verbose_name='shop identifier')),
                ('secret_key', django_pays.models.PasswordField(max_length=64, verbose_name='secret key')),
                ('default_currency', models.CharField(choices=[('CZK', 'CZK'), ('EUR', 'EUR'), ('USD', 'USD')], max_length=3, verbose_name='default currency')),
                ('default_lang', models.CharField(choices=[('CS-CZ', 'CS-CZ'), ('SK-SK', 'SK-SK'), ('EN-US', 'EN-US'), ('RU-RU', 'RU-RU'), ('JA-JP', 'JA-JP')], max_length=5, verbose_name='default language')),
            ],
        ),
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('payment_id', models.CharField(max_length=100, verbose_name='payment identifier')),
                ('order_id', models.CharField(db_index=True, max_length=100, verbose_name='order identifier')),
                ('currency', models.CharField(max_length=3, verbose_name='currency')),
                ('amount', models.BigIntegerField(help_text='amount in the smallest (base) currency unit', verbose_name='amount')),
                ('base_units', models.PositiveIntegerField(help_text='number of base units in currency', verbose_name='base units')),
                ('status', models.PositiveSmallIntegerField(choices=[(2, 'not realized'), (3, 'successful')], verbose_name='payment status')),
                ('status_description', models.TextField(verbose_name='payment status description')),
                ('gateway', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='payments', to='django_pays.Gateway', verbose_name='gateway')),
            ],
            options={
                'verbose_name': 'payment',
                'verbose_name_plural': 'payments',
                'ordering': ('created',),
                'unique_together': {('gateway', 'payment_id')},
            },
        ),
    ]



try:
    from urllib.parse import urlencode
except ImportError:
    # Python 2
    from urllib import urlencode


def payment_url(gateway, order_id, amount, currency=None, email='', lang=None):
    from .models import Gateway

    if not isinstance(gateway, Gateway):
        try:
            gateway = Gateway.objects.get(slug=gateway)
        except Gateway.DoesNotExist:
            return None

    if not currency:
        currency = gateway.default_currency

    return 'https://www.pays.cz/paymentorder?' + urlencode({
        'Merchant': gateway.merchant_id,
        'Shop': gateway.shop_id,
        'MerchantOrderNumber': order_id,
        'Amount': int(amount * Gateway.CURRENCY_BASE_UNITS.get(currency, 100)),
        'Currency': currency,
        'Lang': lang or gateway.default_lang,
        'Email': email,
    })



import hmac

from django.db import transaction
from django.http import HttpResponse
from django.shortcuts import get_object_or_404

from .models import Gateway, Payment


def confirm(request, slug):
    gateway = get_object_or_404(Gateway, slug=slug)
    response = HttpResponse()
    response.status_code = 202

    try:
        msg = (
            '{PaymentOrderID[0]}{MerchantOrderNumber[0]}{PaymentOrderStatusID[0]}'
            '{CurrencyID[0]}{Amount[0]}{CurrencyBaseUnits[0]}'.format(**request.GET)
        )
        assert hmac.compare_digest(
            hmac.new(gateway.secret_key.encode(), msg.encode(), 'md5').hexdigest(),
            request.GET['hash'],
        )
        payment = Payment(
            gateway=gateway,
            payment_id=request.GET['PaymentOrderID'],
            order_id=request.GET['MerchantOrderNumber'],
            currency=request.GET['CurrencyID'],
            amount=int(request.GET['Amount']),
            base_units=int(request.GET['CurrencyBaseUnits']),
            status=int(request.GET['PaymentOrderStatusID']),
            status_description=request.GET['PaymentOrderStatusDescription'],
        )
    except (AssertionError, KeyError, ValueError):
        response.status_code = 400
    else:
        with transaction.atomic():
            payment.save()
    return response



from cms.menu_bases import CMSAttachMenu
from django.core.urlresolvers import NoReverseMatch, reverse
from django.db.models.signals import post_delete, post_save
from django.utils.translation import ugettext_lazy as _
from menus.base import Modifier, NavigationNode
from menus.menu_pool import menu_pool

from .models.subjects import SubjectType
from .utils import current_url, first_upper, url_with_back


@menu_pool.register_menu
class LeprikonMenu(CMSAttachMenu):
    name = _('Leprikon')

    def get_nodes(self, request):
        """
        This method is used to build the menu tree.
        """
        try:
            nodes = []
            nodes.append(NavigationNode(
                _('Log in'),
                reverse('leprikon:user_login'),
                len(nodes),
                attr={'visible_for_authenticated': False, 'add_url_back': True},
            ))
            nodes.append(NavigationNode(
                _('Create account'),
                reverse('leprikon:user_create'),
                len(nodes),
                attr={'visible_for_authenticated': False},
            ))
            nodes.append(NavigationNode(
                _('Reset password'),
                reverse('leprikon:password_reset'),
                len(nodes),
                attr={'visible_for_authenticated': False},
            ))
            nodes.append(NavigationNode(
                _('Summary'),
                reverse('leprikon:summary'),
                len(nodes),
                attr={'visible_for_anonymous': False},
            ))
            nodes.append(NavigationNode(
                _('Reports'),
                reverse('leprikon:report_list'),
                len(nodes),
                attr={'require_staff': True},
            ))
            leader = len(nodes)
            nodes.append(NavigationNode(
                _('Leader'),
                reverse('leprikon:leader_summary'),
                len(nodes),
                attr={'require_leader': True},
            ))
            for subject_type in SubjectType.objects.all():
                nodes.append(NavigationNode(
                    first_upper(_('My {subject_type}').format(subject_type=subject_type.plural)),
                    reverse('leprikon:subject_list_mine', kwargs={'subject_type': subject_type.slug}),
                    len(nodes),
                    parent_id=leader,
                    attr={'require_leader': True},
                ))
            nodes.append(NavigationNode(
                _('Alternating'),
                reverse('leprikon:course_alternating'),
                len(nodes),
                parent_id=leader,
                attr={'require_leader': True},
            ))
            timesheets = len(nodes)
            nodes.append(NavigationNode(
                _('Timesheets'),
                reverse('leprikon:timesheet_list'),
                len(nodes),
                parent_id=leader,
                attr={'require_leader': True},
            ))
            nodes.append(NavigationNode(
                _('Add new entry'),
                reverse('leprikon:timesheetentry_create'),
                len(nodes),
                parent_id=timesheets,
                attr={'require_leader': True, 'add_url_back': True},
            ))
            nodes.append(NavigationNode(
                _('Messages'),
                reverse('leprikon:message_list'),
                len(nodes),
                attr={'visible_for_anonymous': False},
            ))
            nodes.append(NavigationNode(
                _('Registrations'),
                reverse('leprikon:registration_list'),
                len(nodes),
                attr={'visible_for_anonymous': False},
            ))
            nodes.append(NavigationNode(
                _('Payments'),
                reverse('leprikon:payment_list'),
                len(nodes),
                attr={'visible_for_anonymous': False},
            ))
            participants = len(nodes)
            nodes.append(NavigationNode(
                _('Participants'),
                reverse('leprikon:participant_list'),
                len(nodes),
                attr={'visible_for_anonymous': False},
            ))
            nodes.append(NavigationNode(
                _('Add participant'),
                reverse('leprikon:participant_create'),
                len(nodes),
                parent_id=participants,
                attr={'visible_for_anonymous': False, 'add_url_back': True},
            ))
            nodes.append(NavigationNode(
                _('Add parent'),
                reverse('leprikon:parent_create'),
                len(nodes),
                parent_id=participants,
                attr={'visible_for_anonymous': False, 'add_url_back': True},
            ))
            for subject_type in SubjectType.objects.all():
                nodes.append(NavigationNode(
                    first_upper(subject_type.plural),
                    reverse('leprikon:subject_list', kwargs={'subject_type': subject_type.slug}),
                    len(nodes),
                ))
            nodes.append(NavigationNode(
                _('Leaders'),
                reverse('leprikon:leader_list'),
                len(nodes),
            ))
            nodes.append(NavigationNode(
                _('Terms and Conditions'),
                reverse('leprikon:terms_conditions'),
                len(nodes),
            ))
            nodes.append(NavigationNode(
                _('Log out'),
                reverse('leprikon:user_logout'),
                len(nodes),
                attr={'visible_for_anonymous': False},
            ))
            return nodes
        except NoReverseMatch:
            return []


@menu_pool.register_modifier
class LeprikonModifier(Modifier):
    def modify(self, request, nodes, namespace, root_id, post_cut, breadcrumb):
        if post_cut or breadcrumb:
            return nodes
        final = []
        for node in nodes:
            if (
                (node.attr.get('require_leader', False) and not request.leader) or
                (node.attr.get('require_staff', False) and not request.user.is_staff)
            ):
                if node.parent and node in node.parent.children:
                    node.parent.children.remove(node)
                continue
            else:
                if node.attr.get('add_url_back', False):
                    node.url = url_with_back(node.url, current_url(request))
                final.append(node)
        return final


# clear menu cache with each reload
try:
    menu_pool.clear()
except:
    # menu_pool.clear() uses database,
    # but this code may be executed
    # before the database is created
    pass


def invalidate_menu_cache(sender, **kwargs):
    menu_pool.clear()


post_save.connect(invalidate_menu_cache, sender=SubjectType)
post_delete.connect(invalidate_menu_cache, sender=SubjectType)



# Generated by Django 1.11.12 on 2018-08-29 21:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0018_citizenship_cleanup'),
    ]

    operations = [
        migrations.CreateModel(
            name='Department',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('code', models.PositiveSmallIntegerField(blank=True, default=0, verbose_name='department code')),
            ],
            options={
                'ordering': ('code', 'name'),
                'verbose_name': 'department',
                'verbose_name_plural': 'departments',
            },
        ),
        migrations.RemoveField(
            model_name='manager',
            name='school_years',
        ),
        migrations.RemoveField(
            model_name='manager',
            name='user',
        ),
        migrations.AlterModelOptions(
            name='course',
            options={'ordering': ('code', 'name'), 'verbose_name': 'course', 'verbose_name_plural': 'courses'},
        ),
        migrations.AlterModelOptions(
            name='subject',
            options={'ordering': ('code', 'name'), 'verbose_name': 'subject', 'verbose_name_plural': 'subjects'},
        ),
        migrations.RemoveField(
            model_name='courselistplugin',
            name='managers',
        ),
        migrations.RemoveField(
            model_name='eventlistplugin',
            name='managers',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='managers',
        ),
        migrations.AlterField(
            model_name='subject',
            name='age_groups',
            field=models.ManyToManyField(related_name='subjects', to='leprikon.AgeGroup', verbose_name='age groups'),
        ),
        migrations.DeleteModel(
            name='Manager',
        ),
        migrations.AddField(
            model_name='courselistplugin',
            name='departments',
            field=models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by departments.', related_name='_courselistplugin_departments_+', to='leprikon.Department', verbose_name='departments'),
        ),
        migrations.AddField(
            model_name='eventlistplugin',
            name='departments',
            field=models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by departments.', related_name='_eventlistplugin_departments_+', to='leprikon.Department', verbose_name='departments'),
        ),
        migrations.AddField(
            model_name='subject',
            name='department',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subjects', to='leprikon.Department', verbose_name='department'),
        ),
    ]



from django import forms

from .form import FormMixin


class ConfirmForm(FormMixin, forms.Form):
    answer = forms.BooleanField()

    def __init__(self, *args, **kwargs):
        self.instance = kwargs.pop('instance', None)
        super(ConfirmForm, self).__init__(*args, **kwargs)

    def save(self):
        return self.instance



import django
from django.contrib import admin
from django.contrib.sites.models import Site
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from .models import Gateway, Payment


class IdentifiedFieldListFilter(admin.RelatedFieldListFilter):
    def choices(self, changelist):
        yield {
            'selected': self.lookup_val_isnull is None,
            'query_string': changelist.get_query_string(remove=[self.lookup_kwarg_isnull]),
            'display': _('All'),
        }
        yield {
            'selected': self.lookup_val_isnull == 'True',
            'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'True'}),
            'display': _('Not identified'),
        }
        yield {
            'selected': self.lookup_val_isnull == 'False',
            'query_string': changelist.get_query_string({self.lookup_kwarg_isnull: 'False'}),
            'display': _('Identified'),
        }


@admin.register(Gateway)
class GatewayAdmin(admin.ModelAdmin):
    list_display = ('name', 'confirmation_url')

    def confirmation_url(self, obj):
        return 'https://{domain}{url}'.format(
            domain=Site.objects.get_current().domain,
            url=reverse('pays:confirm', args=(obj.slug,)),
        )
    confirmation_url.short_description = _('confirmation url')

    def get_readonly_fields(self, request, obj=None):
        # do not change secret key if not given
        if request.method == 'POST' and not request.POST.get('secret_key'):
            return ('secret_key',)
        else:
            return ()


@admin.register(Payment)
class PaymentAdmin(admin.ModelAdmin):
    date_hierarchy = 'created'
    list_display = tuple(
        ('account_name' if f.name == 'account' else f.name) for f in Payment._meta.fields
    )[1:] + tuple(
        r.name for r in Payment._meta.related_objects
    )
    list_filter = ('gateway',) + tuple(
        (r.name, IdentifiedFieldListFilter)
        for r in Payment._meta.related_objects
    )

    def has_add_permission(self, request):
        return False

    # read only
    if django.VERSION > (2,):
        def has_change_permission(self, request, obj=None):
            return False
    else:
        def get_readonly_fields(self, request, obj=None):
            return tuple(
                f.name for f in self.model._meta.fields
                if not f.primary_key
            ) if obj else ()



from django import forms
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from ..models.schoolyear import SchoolYear


class SchoolYearForm(forms.Form):
    school_year = forms.ModelChoiceField(
        label       = _('School year'),
        queryset    = SchoolYear.objects.all(),
        empty_label = None,
    )
    back = settings.LEPRIKON_PARAM_BACK

    def __init__(self, request, *args, **kwargs):
        kwargs['initial'] = {'school_year': request.school_year}
        super(SchoolYearForm, self).__init__(*args, **kwargs)
        if not request.user.is_staff:
            self.fields['school_year'].queryset = (
                self.fields['school_year'].queryset.filter(active=True)
            )



from django import forms
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from ..models.events import Event, EventRegistration
from ..models.schoolyear import SchoolYear
from ..models.subjects import SubjectType
from ..utils import currency
from .filters import (
    ApprovedListFilter, CanceledListFilter, EventGroupListFilter,
    EventListFilter, EventTypeListFilter, LeaderListFilter,
    SchoolYearListFilter,
)
from .pdf import PdfExportAdminMixin
from .subjects import (
    SubjectBaseAdmin, SubjectPaymentBaseAdmin, SubjectRegistrationBaseAdmin,
)


class EventAdmin(SubjectBaseAdmin):
    subject_type = SubjectType.EVENT
    registration_model = EventRegistration
    list_display    = (
        'id', 'code', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list',
        'get_times_list',
        'place', 'public', 'registration_allowed_icon',
        'get_registrations_link',
        'icon', 'note',
    )
    list_export     = (
        'id', 'code', 'name', 'department', 'subject_type', 'get_groups_list', 'get_leaders_list',
        'start_date', 'start_time', 'end_date', 'end_time',
        'place', 'public',
        'get_approved_registrations_count', 'get_unapproved_registrations_count', 'note',
    )
    list_filter     = (
        ('school_year',     SchoolYearListFilter),
        'department',
        ('subject_type',    EventTypeListFilter),
        ('groups',          EventGroupListFilter),
        ('leaders',         LeaderListFilter),
    )
    date_hierarchy  = 'start_date'
    actions         = (
        'publish', 'unpublish',
        'copy_to_school_year',
    )

    def publish(self, request, queryset):
        Event.objects.filter(id__in=[reg['id'] for reg in queryset.values('id')]).update(public = True)
        self.message_user(request, _('Selected events were published.'))
    publish.short_description = _('Publish selected events')

    def unpublish(self, request, queryset):
        Event.objects.filter(id__in=[reg['id'] for reg in queryset.values('id')]).update(public = False)
        self.message_user(request, _('Selected events were unpublished.'))
    unpublish.short_description = _('Unpublish selected events')

    def copy_to_school_year(self, request, queryset):
        class SchoolYearForm(forms.Form):
            school_year = forms.ModelChoiceField(
                label=_('Target school year'),
                help_text=_('All selected events will be copied to selected school year.'),
                queryset=SchoolYear.objects.all(),
            )
        if request.POST.get('post', 'no') == 'yes':
            form = SchoolYearForm(request.POST)
            if form.is_valid():
                school_year = form.cleaned_data['school_year']
                for event in queryset.all():
                    event.copy_to_school_year(school_year)
                self.message_user(request, _('Selected events were copied to school year {}.').format(school_year))
                return
        else:
            form = SchoolYearForm()
        return render(
            request,
            'leprikon/admin/action_form.html',
            {
                'title': _('Select target school year'),
                'queryset': queryset,
                'opts': self.model._meta,
                'form': form,
                'action': 'copy_to_school_year',
                'action_checkbox_name': admin.helpers.ACTION_CHECKBOX_NAME,
            },
        )
    copy_to_school_year.short_description = _('Copy selected events to another school year')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_registrations__subject__in = queryset
        ).distinct()



class EventRegistrationAdmin(PdfExportAdminMixin, SubjectRegistrationBaseAdmin):
    list_display    = (
        'variable_symbol', 'download_tag', 'subject_name', 'participant', 'price', 'event_discounts', 'event_payments',
        'created', 'approved', 'payment_requested', 'cancel_request', 'canceled', 'note',
    )
    list_filter     = (
        ('subject__school_year',    SchoolYearListFilter),
        'subject__department',
        ('subject__subject_type',   EventTypeListFilter),
        ApprovedListFilter,
        CanceledListFilter,
        ('subject',                 EventListFilter),
        ('subject__leaders',        LeaderListFilter),
    )

    def event_discounts(self, obj):
        status = obj.get_payment_status()
        return format_html(
            '<a href="{href_list}"><b>{amount}</b></a>'
            ' &nbsp; <a class="popup-link" href="{href_add}" style="background-position: 0 0" title="{title_add}">'
            '<img src="{icon_add}" alt="+"/></a>',
            href_list   = reverse('admin:leprikon_eventdiscount_changelist') + '?registration={}'.format(obj.id),
            amount      = currency(status.discount),
            href_add    = reverse('admin:leprikon_eventdiscount_add') + '?registration={}'.format(obj.id),
            icon_add    = static('admin/img/icon-addlink.svg'),
            title_add   = _('add discount'),
        )
    event_discounts.allow_tags = True
    event_discounts.short_description = _('event discounts')

    def event_payments(self, obj):
        status = obj.get_payment_status()
        return format_html(
            '<a class="popup-link" style="color: {color}" href="{href_list}" title="{title}"><b>{amount}</b></a>'
            ' &nbsp; <a class="popup-link" href="{href_add}" style="background-position: 0 0" title="{title_add}">'
            '<img src="{icon_add}" alt="+"/></a>',
            color       = status.color,
            href_list   = reverse('admin:leprikon_subjectpayment_changelist') + '?registration={}'.format(obj.id),
            title       = status.title,
            amount      = currency(status.paid),
            href_add    = reverse('admin:leprikon_subjectpayment_add') + '?registration={}'.format(obj.id),
            icon_add    = static('admin/img/icon-addlink.svg'),
            title_add   = _('add payment'),
        )
    event_payments.allow_tags = True
    event_payments.short_description = _('event payments')



class EventDiscountAdmin(PdfExportAdminMixin, SubjectPaymentBaseAdmin):
    list_display    = ('accounted', 'registration', 'subject', 'amount_html', 'explanation')
    list_export     = ('accounted', 'registration', 'subject', 'amount', 'explanation')
